/*** cli.l -- unserding CLI scanner  -*- C -*-
 *
 * Copyright (C) 2002-2008 Sebastian Freundt
 *
 * Author:  Sebastian Freundt <freundt@math.tu-berlin.de>
 *
 * This file is part of QaoS.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * 3. Neither the name of the author nor the names of any contributors
 *    may be used to endorse or promote products derived from this
 *    software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR "AS IS" AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 * IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 ***/

%{
#define YY_EXTRA_TYPE	void*
%}

%option reentrant
%option noyywrap
%option warn
%option yylineno
%option nounput
%option noinput
%option bison-bridge
%option header-file="lex.yy.h"
%option outfile="lex.yy.c"
%option pointer
%option nodebug

%{

#include <stdlib.h>
#include <string.h>

#include "cli-common.h"
#include "unsercli-unsercli.h"
#include "unserding-dbg.h"

extern int cli_yyget_column(yyscan_t);
extern void cli_yyset_column(int, yyscan_t);

#define RETURN_TOKEN(_x)			\
	do {					\
		UD_DEBUG(#_x "\n");		\
		return _x;			\
	} while (0)

/* turn off tedious warnings */
#if defined __INTEL_COMPILER
# pragma warning (disable:593)
# pragma warning (disable:869)
# pragma warning (disable:1419)
# pragma warning (disable:2259)
#elif defined __GNUC__
# pragma GCC diagnostic ignored "-Wswitch-default"
# pragma GCC diagnostic ignored "-Wsign-compare"
# pragma GCC diagnostic ignored "-Wunused-parameter"
#endif	/* __GNUC__ || __INTEL_COMPILER */
%}

ws		[ \t\f\n\r]+
geq		(>=)|(=>)
leq		(<=)|(=<)
neq		(<>)|(!=)
less		<
grtr		>
eq		[:=]?|is
neg		!

key		:[a-z]+
val		[A-Za-z0-9,.-]+

int		[-+]?[0-9]+
hexdgt		[0-9a-fA-F]
intdgt		[0-9]

abs		\|

%x STRING
%x ARGS

%%
<ARGS>\"	{ BEGIN(STRING); }
<STRING>[^\"]*\"	{
	BEGIN(INITIAL);
	yylval->slen = cli_yyget_leng(yyscanner) - /* the final quote */ 1;
	yylval->sval = cli_yyget_text(yyscanner);
	RETURN_TOKEN(TOK_STRING);
}

{hexdgt}{4,4}	{
	BEGIN(ARGS);
	yylval->slen = cli_yyget_leng(yyscanner);
	yylval->sval = cli_yyget_text(yyscanner);
	RETURN_TOKEN(TOK_HEXCMD);
}	

h?elp		{ BEGIN(ARGS); RETURN_TOKEN(TOK_WTF); }
wtf		{ BEGIN(ARGS); RETURN_TOKEN(TOK_WTF); }
cya		{ BEGIN(ARGS); RETURN_TOKEN(TOK_CYA); }
bye		{ BEGIN(ARGS); RETURN_TOKEN(TOK_CYA); }
kthx		{ BEGIN(ARGS); RETURN_TOKEN(TOK_CYA); }
kthxbye		{ BEGIN(ARGS); RETURN_TOKEN(TOK_CYA); }
quit		{ BEGIN(ARGS); RETURN_TOKEN(TOK_CYA); }
logout		{ BEGIN(ARGS); RETURN_TOKEN(TOK_CYA); }
alias		{ BEGIN(ARGS); RETURN_TOKEN(TOK_ALI); }

<ARGS>0x{hexdgt}{9,16}	{ 
	yylval->slen = cli_yyget_leng(yyscanner);
	yylval->sval = cli_yyget_text(yyscanner);
	RETURN_TOKEN(TOK_UI64H);
}

<ARGS>0x{hexdgt}{5,8}	{ 
	yylval->slen = cli_yyget_leng(yyscanner);
	yylval->sval = cli_yyget_text(yyscanner);
	RETURN_TOKEN(TOK_UI32H);
}

<ARGS>0x{hexdgt}{3,4}	{ 
	yylval->slen = cli_yyget_leng(yyscanner);
	yylval->sval = cli_yyget_text(yyscanner);
	RETURN_TOKEN(TOK_UI16H);
}

<ARGS>0x{hexdgt}{1,2}	{ 
	yylval->slen = cli_yyget_leng(yyscanner);
	yylval->sval = cli_yyget_text(yyscanner);
	RETURN_TOKEN(TOK_UI8H);
}

<ARGS>[-+]?{intdgt}+[uU]	{
	yylval->slen = cli_yyget_leng(yyscanner);
	yylval->sval = cli_yyget_text(yyscanner);
	RETURN_TOKEN(TOK_UINT);
}

<ARGS>[-+]?{intdgt}+	{
	yylval->slen = cli_yyget_leng(yyscanner);
	yylval->sval = cli_yyget_text(yyscanner);
	RETURN_TOKEN(TOK_INT);
}

 /* don't do that at the moment, we can parse the string based on the context
  * ourselves, and much more efficiently */
 /*{int}		{ RETURN_TOKEN(TOK_INTEGER); }*/
<ARGS>{key}	{
	yylval->slen = cli_yyget_leng(yyscanner);
	yylval->sval = cli_yyget_text(yyscanner);
	RETURN_TOKEN(TOK_KEY);
}
<ARGS>{val}	{
	yylval->slen = cli_yyget_leng(yyscanner);
	yylval->sval = cli_yyget_text(yyscanner);
	RETURN_TOKEN(TOK_VAL);
}

 /* more ignored shit, whitespace et al */
<*>{ws}		{ /* ignore */ }

%%

/* cli.l ends here */
